#include "emacs-module.h"
#include <tree_sitter/parser.h>

int plugin_is_GPL_compatible;

extern TSLanguage *tree_sitter_json(void);

/* REF: http://diobla.info/blog-archive/modules-tut.html  */

static emacs_value
Ftree_sitter_json_test
(emacs_env *env, ptrdiff_t nargs, emacs_value args[], void *data)
{
  TSLanguage *language = tree_sitter_json();
  return env->make_user_ptr (env, NULL, language);
}

/* Bind NAME to FUN.  */
static void
bind_function (emacs_env *env, const char *name, emacs_value Sfun)
{
  /* Set the function cell of the symbol named NAME to SFUN using
     the 'fset' function.  */

  /* Convert the strings to symbols by interning them */
  emacs_value Qfset = env->intern (env, "fset");
  emacs_value Qsym = env->intern (env, name);

  /* Prepare the arguments array */
  emacs_value args[] = { Qsym, Sfun };

  /* Make the call (2 == nb of arguments) */
  env->funcall (env, Qfset, 2, args);
}

/* Provide FEATURE to Emacs.  */
static void
provide (emacs_env *env, const char *feature)
{
  /* call 'provide' with FEATURE converted to a symbol */

  emacs_value Qfeat = env->intern (env, feature);
  emacs_value Qprovide = env->intern (env, "provide");
  emacs_value args[] = { Qfeat };

  env->funcall (env, Qprovide, 1, args);
}

int
emacs_module_init (struct emacs_runtime *ert)
{
  emacs_env *env = ert->get_environment (ert);

  /* create a lambda (returns an emacs_value) */
  emacs_value fun = env->make_function (env,
              0,            /* min. number of arguments */
              0,            /* max. number of arguments */
              &Ftree_sitter_json_test,  /* actual function pointer */
              "doc",        /* docstring */
              NULL          /* user pointer of your choice (data param in Fmymod_test) */
  );

  bind_function (env, "tree-sitter-json", fun);
  provide (env, "tree-sitter-json");

  /* loaded successfully */
  return 0;
}
